# -*- coding: utf-8 -*-

import json
import tensorflow as tf
# conda install -n py38 tensorflow
from tensorflow import keras
# 从tensorflow.keras获取想要的API
from tensorflow.keras.preprocessing.text import Tokenizer
# 可以用pad_sequences来填充数列
from tensorflow.keras.preprocessing.sequence import pad_sequences

# 分词保留的最大单词数
vocab_size = 10000
# 每个词创建的向量维度
embedding_dim = 16
# 数据最大长度
max_length = 100
# 数据长度大于所设定句子长度maxlen，可加入截断参数，truncating='post'，
trunc_type = 'post'
# 在句子后边补齐
padding_type = 'post'
# oov_token属性，将语料库中没有的单词标记出来
oov_tok = "<OOV>"
# 两万训练数据
training_size = 20000



sentences = []
labels = []
# urls = []

file = open("data/Sarcasm_Headlines_Dataset.json", 'r')

for line in file.readlines():
    datastore = json.loads(line)
    sentences.append(datastore['headline'])
    labels.append(datastore['is_sarcastic'])
    # urls.append(datastore['article_link'])

# with open("data/sarcasm.json", 'r') as f:
#     datastore = json.load(f)
#
#
#
# for item in datastore:
#     sentences.append(item['headline'])
#     labels.append(item['is_sarcastic'])
#     # urls.append(item['article_link'])

# 进行分片，以用于训练集和测试集
training_sentences = sentences[0:training_size]
testing_sentences = sentences[training_size:]
training_labels = labels[0:training_size]
testing_labels = labels[training_size:]

# 要保证创建的神经网络只见过训练集，没有见过训练集
# 分词保留的最大单词数，为了显示语料库中没有的单词，利用oov_token属性，将其设置为语料库中无法识别的内容
# 实例化分词器
tokenizer = Tokenizer(num_words=vocab_size, oov_token=oov_tok)
# 用这个训练集创建token字典
tokenizer.fit_on_texts(training_sentences)
# 分词并编号
word_index = tokenizer.word_index

# 每个句子的token序列
training_sequences = tokenizer.texts_to_sequences(training_sentences)
# 填充序列变为同样长度，以最长的为基准
# 如果添加参数 padding='post'，代表在token序列后方加入0补齐
# 也可以加入参数maxlen，指定所需句子长度，如 maxlen=5
# 如果数据长度大于所设定句子长度maxlen，可加入截断参数，truncating='post'，
training_padded = pad_sequences(training_sequences, maxlen=max_length,
                                padding=padding_type, truncating=trunc_type)

testing_sequences = tokenizer.texts_to_sequences(testing_sentences)
testing_padded = pad_sequences(testing_sequences, maxlen=max_length,
                               padding=padding_type, truncating=trunc_type)
# numpy一个科学计算库
import numpy as np

# np.array将数组等其他形式转化为numpy数组，以便进行矩阵等运算
training_padded_after = np.array(training_padded)
training_labels_after = np.array(training_labels)
testing_padded_after = np.array(testing_padded)
testing_labels_after = np.array(testing_labels)

# 建立模型
model = tf.keras.Sequential([
    # 每个单词的情感方向都会被一次又一次的学习
    tf.keras.layers.Embedding(vocab_size, embedding_dim, input_length=max_length),
    # 将向量进行相加
    tf.keras.layers.GlobalAveragePooling1D(),

    # 然后嵌入到一个普通的神经网络中，24输出维度大小units（该层有多少个神经元），activation激活函数
    # 输入层和隐层，输出层等等
    # https://blog.csdn.net/ybdesire/article/details/85217688
    tf.keras.layers.Dense(24, activation='relu'),
    tf.keras.layers.Dense(1, activation='sigmoid')
])
# 查看https://blog.csdn.net/chaojishuai123/article/details/114580892
# loss目标函数，'binary_crossentropy'对数损失；metrics评价函数（acc和val_acc就是通过定义评价函数得到的）
model.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])

# 通过model.summary()输出模型各层的参数情况
model.summary()

# 30次迭代训练
num_epochs = 30
# 训练以及检测
# training_padded训练集输入特征，training_labels训练集标签
# epochs迭代次数，validation_data = (测试集的输入特征，测试集的标签），verbose：日志显示，0为不在标准输出流输出日志信息，1为输出进度条记录，2为每个epoch输出一行记录
# https://blog.csdn.net/LuYi_WeiLin/article/details/88555813

# print(type(testing_padded_after))
# print(training_padded_after[0:9])
history = model.fit(training_padded_after, training_labels_after,
                    epochs=num_epochs,
                    validation_data=(testing_padded_after, testing_labels_after),
                    verbose=2)

# conda install -n py38 tensorflow
# https://zhuanlan.zhihu.com/p/352483773,引入matlab包

import matplotlib.pyplot as plt


# 画图
def plot_graphs(history, string):
    plt.plot(history.history[string])
    plt.plot(history.history['val_' + string])
    plt.xlabel("Epochs")
    plt.ylabel(string)
    plt.legend([string, 'val_' + string])
    plt.show()


plot_graphs(history, "accuracy")
plot_graphs(history, "loss")

# 将value和key转化，可以互相反查
reverse_word_index = dict([(value, key) for (key, value) in word_index.items()])


def decode_sentence(text):
    # 字典dict.get,返回指定键i的值，如果没有返回'？'。
    # join用法，把？，用空格连接
    return ' '.join([reverse_word_index.get(i, '?') for i in text])

# 测试一下对不对
print('training_padded[0]为：',training_padded[0])
print(decode_sentence(training_padded[0]))
print(training_sentences[2])
print(labels[2])

e = model.layers[0]
# 返回该层的权重
weights = e.get_weights()[0]
print(weights.shape)  # shape: (vocab_size, embedding_dim)

# import io
#
# out_v = io.open('vecs.tsv', 'w', encoding='utf-8')
# out_m = io.open('meta.tsv', 'w', encoding='utf-8')
# for word_num in range(1, vocab_size):
#     word = reverse_word_index[word_num]
#     embeddings = weights[word_num]
#     out_m.write(word + "\n")
#     out_v.write('\t'.join([str(x) for x in embeddings]) + "\n")
# out_v.close()
# out_m.close()
#
# try:
#     from google.colab import files
# except ImportError:
#     pass
# else:
#     files.download('vecs.tsv')
#     files.download('meta.tsv')

# 利用神经网络，判断新句子的情感色彩
new_sentence = ["granny starting to fear spiders in the garden might be real",
                "game of thrones season finale showing this sunday night"]
new_sequences = tokenizer.texts_to_sequences(new_sentence)

padded = pad_sequences(new_sequences, maxlen=max_length,
                       padding=padding_type,
                       truncating=trunc_type)
print(model.predict(padded))
